Imports System.Collections.ObjectModel
Imports System.ComponentModel

Public Class frmNetAuflistungen

    Private _innerList As New List(Of DataClass)

    ' BindingList(Of T) feuert spezifische Events, die fr DataBinding 
    '  erforderlich sind.
    Private _BindingList As New BindingList(Of DataClass)(_innerList)

    ' ReadOnlyCollection(Of T) ist ntzlich, wenn man eine Auflistung nur mit 
    '  LeseZugriff offenlegen will. Die in dieser Anwendung ausgelsten 
    '  NotSupportedExceptions demonstrieren dieses
    Private _ReadonlyList As New ReadOnlyCollection(Of DataClass)(_innerList)

    ' Diese ActionList beerbt die Collection(Of DataClass). Dadurch kann sie 
    '  bei Hinzufgen, Lschen etc. benutzerdefinierte Aktionen auszufhren
    Private _ActionList As New ActionList(_innerList)

    'die verschiedenen Listen in ein Array stopfen
    Private _Sources() As IList(Of DataClass) = New IList(Of DataClass)() { _
       _innerList, _BindingList, _ReadonlyList, _ActionList}

    Private Sub frmNetAuflistungen_Load(ByVal sender As Object, _
            ByVal e As EventArgs) Handles MyBase.Load

        ReInit()

        ' Auswahl der Listen als Datasource des DatagridViews einrichten
        For Each lst As Object In _Sources
            cmbSelectSource.Items.Add(lst.GetType.Name)
        Next
        cmbSelectSource.SelectedIndex = 0
    End Sub

    Private Sub cmbSelectSource_SelectedIndexChanged(ByVal sender As Object, _
            ByVal e As EventArgs) Handles cmbSelectSource.SelectedIndexChanged

        BindingSource.DataSource = _Sources(cmbSelectSource.SelectedIndex)
    End Sub

    ' Helfer fr die Sortierung
    Private Function CompareDataClass(ByVal x As DataClass, _
            ByVal y As DataClass) As Integer

        Return x.Name.CompareTo(y.Name)
    End Function

    Private Sub Button_Click(ByVal sender As Object, _
            ByVal e As EventArgs) Handles btInsert.Click, btRemove.Click, _
            btExchange.Click, btSort.Click, btReset.Click, btResetBinding.Click

        ' Die meisten der hier angebotenen Aktionen spiegeln sich erst dann im 
        '  DatagridView wieder, wenn ein ResetBinding erfolgt
        StatusLabel.Text = "ResetBinding erforderlich"
        Select Case True
            Case sender Is btReset
                ReInit()

            Case sender Is btSort
                ' Interessant am Wrapper-Prinzip ist, da auf der inneren Liste 
                '  immer noch Operationen ausfhrbar sind, die der Wrapper 
                '  selbst nicht (mehr) untersttzt. Ich kann so also die 
                '  Elemente einer ReadOnlyCollection quasi "im Hintergrund" 
                '  sortieren, mit Auswirkung auch beim "Empfnger", dem solches 
                '  verwehrt ist (weil er keinen Zugriff auf die InnerList hat).
                '  Hingegen eine BindingList direkt zu sortieren wre sehr 
                '  unperformant, da ja jede einzelne nderung an die 
                '  angebundenen Controls gemeldet wrde
                _innerList.Sort(AddressOf CompareDataClass)

            Case sender Is btResetBinding
                BindingSource.ResetBindings(False)
                StatusLabel.Text = ""

            Case Else
                If BindingSource.DataSource Is _BindingList Then _
                    StatusLabel.Text = ""

                Dim lst As IList(Of DataClass) = _
                   DirectCast(BindingSource.DataSource, IList(Of DataClass))
                Dim data As DataClass = lst(BindingSource.Position)
                Select Case True
                    Case sender Is btExchange
                        lst(BindingSource.Position) = _
                            New DataClass(data.Name & " " & lst.Count)

                    Case sender Is btRemove
                        lst.Remove(data)

                    Case sender Is btInsert
                        lst.Insert(BindingSource.Position, _
                            New DataClass("newItem" & lst.Count))
                End Select
        End Select
    End Sub

    Private Sub ReInit()
        Dim rnd As New Random(33)
        _innerList.Clear()
        For I As Integer = 0 To 15
            _innerList.Add(New DataClass("Data " & rnd.Next(0, 30)))
        Next
    End Sub

End Class
